/*
 * init.c	version of 2009/11/18
 */

#include "model.h"

extern Firm firm[NFirm],obs[NFirm];
extern Worker worker[NWorker];
extern Holder holder[NHolder];
extern Env env;
extern long seed;

void ini_agent(){
	int i,t;
	double tmp,tmpi,nf,nw,nh;

	nf = (double)(NFirm);
	nw = (double)(NWorker);
	nh = (double)(NHolder);

	env.marginal = 1.0;	// effect on marginal cost
	env.upcsr = 0.1;	// s_1
	env.depr = 0.1;		// s_3
	env.beta = 0.1;		// s_2
	env.iner = 1.0;		// s_5
	env.eff = 0.0;		// s_4
	env.abl = 1.0;		// A
	env.ablr = 0.1;		// Range of A
	env.iunit = 1.0;	// investment unit

	tmp = nw / nf * (env.eff + env.abl) + nh / nf / env.depr * env.beta;
	for(i=0;i<NFirm;i++){
		tmpi = tmp * (1.0 - env.upcsr * firm[i].csr) * env.marginal;
/* homogeneous version
		firm[i].csr = 1.0;
		firm[i].quantity[0] = 1000.0;
		firm[i].wage[0] = 1000.0;
		firm[i].demand[0] = 1000.0;
		firm[i].invest[0] = 1.0;
		firm[i].stock[0] = 1.0;
*/
// /* heterogeneous version
		firm[i].csr = (double)(i) / nf;
		firm[i].quantity[0] = tmpi;
		firm[i].wage[0] = tmpi / nw * nf;
		firm[i].demand[0] = tmpi;
		firm[i].invest[0] = nh / nf;
		firm[i].stock[0] = nh / nf / env.depr;
// */
	}
	for(i=0;i<NWorker;i++){
		worker[i].abl = (ran1(&seed) * 2.0 - 1.0) * env.ablr + env.abl;
		worker[i].csrW = ran1(&seed);
//		worker[i].csrW = 0.0;
		worker[i].csrC = ran1(&seed);
//		worker[i].csrC = 0.0;
	}
	for(i=0;i<NHolder;i++){
		holder[i].csr = ran1(&seed);
//		holder[i].csr = 0.0;
		holder[i].invest = env.iunit;		// Fix
	}

}

void initialize(int r){
	seed = -5L * (r+1);
	ran1(&seed);	// Initialising a random seed
	ini_agent();
}

void ini_ran(){
	int i,t;
	for(i=0;i<NFirm;i++){
		obs[i].csr = 0.0;
		for(t=0;t<Time;t++) obs[i].quantity[t] = 0.0;
		for(t=0;t<Time;t++) obs[i].wage[t] = 0.0;
		for(t=0;t<Time;t++) obs[i].demand[t] = 0.0;
		for(t=0;t<Time;t++) obs[i].invest[t] = 0.0;
		for(t=0;t<Time;t++) obs[i].stock[t] = 0.0;
	}
}
void calObsF(int r){
	int i,t;
	for(i=0;i<NFirm;i++){
		(obs[i].csr) += (firm[i].csr);
		for(t=0;t<Time;t++) (obs[i].quantity[t]) += (firm[i].quantity[t]);
		for(t=0;t<Time;t++) (obs[i].wage[t]) += (firm[i].wage[t]);
		for(t=0;t<Time;t++) (obs[i].demand[t]) += (firm[i].demand[t]);
		for(t=0;t<Time;t++) (obs[i].invest[t]) += (firm[i].invest[t]);
		for(t=0;t<Time;t++) (obs[i].stock[t]) += (firm[i].stock[t]);
	}
}

void printResF(){

// Final Status
	int i,t,t1;
	t1 = Time - 1;
	for(i=0;i<NFirm;i++){
		(obs[i].csr) /= Random;
		for(t=0;t<Time;t++) {
			(obs[i].quantity[t]) /= Random;
			(obs[i].wage[t]) /= Random;
			(obs[i].demand[t]) /= Random;
			(obs[i].invest[t]) /= Random;
			(obs[i].stock[t]) /= Random;
		}
	}
	printf("CSR,Quantity,Wage,Demand,Invest,Stock\n");
	for(i=0;i<NFirm;i++){
		printf("%5.3f",obs[i].csr);
		printf(",%5.3f",obs[i].quantity[t1]);
		printf(",%5.3f",obs[i].wage[t1]);
		printf(",%5.3f",obs[i].demand[t1]);
		printf(",%5.3f",obs[i].invest[t1]);
		printf(",%5.3f",obs[i].stock[t1]);
		printf("\n");
	}

/*
// transitions
	int i,t;
	for(i=0;i<NFirm;i++){
		(obs[i].csr) /= Random;
		for(t=0;t<Time;t++) {
			(obs[i].quantity[t]) /= Random;
			(obs[i].wage[t]) /= Random;
			(obs[i].demand[t]) /= Random;
			(obs[i].invest[t]) /= Random;
			(obs[i].stock[t]) /= Random;
		}
	}
	printf("time");
	for(i=0;i<NFirm;i++) printf(",CSR(%.1f)",obs[i].csr);
	printf("\n");
	for(t=0;t<Time;t++){
		printf("%2d",t);
		for(i=0;i<NFirm;i++) printf(",%5.1f",obs[i].stock[t]);
		printf("\n");
	}
*/
/*
// homogeneous version
	int i,t;
	Firm unit;
	for(i=0;i<NFirm;i++){
		(obs[i].csr) /= Random;
		for(t=0;t<Time;t++) {
			(obs[i].quantity[t]) /= Random;
			(obs[i].wage[t]) /= Random;
			(obs[i].demand[t]) /= Random;
			(obs[i].invest[t]) /= Random;
			(obs[i].stock[t]) /= Random;
		}
	}
	unit.csr = 0.0;
	for(t=0;t<Time;t++){
		unit.quantity[t] = 0.0;
		unit.wage[t] = 0.0;
		unit.demand[t] = 0.0;
		unit.invest[t] = 0.0;
		unit.stock[t] = 0.0;
	}
	for(i=0;i<NFirm;i++){
		(unit.csr) += (obs[i].csr);
		for(t=0;t<Time;t++) {
			(unit.quantity[t]) += (obs[i].quantity[t]);
			(unit.wage[t]) += (obs[i].wage[t]);
			(unit.demand[t]) += (obs[i].demand[t]);
			(unit.invest[t]) += (obs[i].invest[t]);
			(unit.stock[t]) += (obs[i].stock[t]);
		}
	}
	unit.csr /= NFirm;
	for(t=0;t<Time;t++){
		(unit.quantity[t]) /= NFirm;
		(unit.wage[t]) /= NFirm;
		(unit.demand[t]) /= NFirm;
		(unit.invest[t]) /= NFirm;
		(unit.stock[t]) /= NFirm;
	}
	printf("time,Q,W,X,I,K\n");
	for(t=0;t<Time;t++){
		printf("%2d",t);
		printf(",%.1f",unit.quantity[t]);
		printf(",%.1f",unit.wage[t]);
		printf(",%.1f",unit.demand[t]);
		printf(",%.1f",unit.invest[t]);
		printf(",%.1f",unit.stock[t]);
		printf("\n");
	}
*/
}
