/*
 * model.h (version of the basic CSR model)
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#define Random	100	// No. of random seeds
#define NFirm	10	// No. of firms
#define NWorker	1000	// No. of workers = No. of consumers
#define NHolder	1000	// No. of shareholders
#define Time	100	// Simulation periods

typedef struct _env{
	double marginal;	// Effect on marginal production cost
	double upcsr;	// s1, Upper limit of CSR activity in firms performance
	double depr;	// s3, Depreciation Ratio = 0.1
	double beta;	// s2, Ratio of labor force to stock on production function
	double iner;	// s5, Inertia quantity for value functions for firm
	double eff;	// s4, Standard level of effort
	double abl;	// Standard level of ability
	double ablr;	// Range of ability
	double iunit;	// Investment unit
} Env;

typedef struct _firm{
	double csr;	// Degree of CSR [0,1)
	double quantity[Time];	// Product Quantity
	double wage[Time];	// Wage
	double demand[Time];	// Demand of Goods
	double invest[Time];	// Investment
	double stock[Time];	// Capital Stock
} Firm;

typedef struct _worker{
	double abl;	// Ability
	double csrW;	// Awareness for CSR [0,1) as Worker
	double csrC;	// as Consumer
	double effort[Time];	// Effort Level
	double wage[Time];	// Wage
	int choice[Time];	// Firm No to be hired
	int consump[Time];	// Firm No to buy
} Worker;

typedef struct _holder{
	double csr;	// Awareness for CSR [0,1)
	double invest;	// Investment for Firm = 1.0
	int choice[Time];	// Firm No. to invest
} Holder;

/* init.c */
void ini_agent(void);	// Initialising Agents
void initialize(int);	// All initialisings
void ini_ran(void);	// Initialising observation procedures
void calObsF(int);	// Calculations for observation
void printResF(void);	// Output

/* phase.c */
void preF(int);		// Preprocess
void investF(int);	// Investment of Shareholders
void jobF(int);		// Job Marketing
void productF(int);	// Product by firms
void consumpF(int);	// Consumption
void cutCsrF(void);	// Cutting CSR awareness

/* funct.c */
int choiceVecF(double []);	// Choose one element from a weighted vector
double maxF(double,double);

/* ran1.c */
double ran1(long *);	// Random generator [0,1)

