/*
 * phase.c
 */

#include "model.h"

extern Firm firm[NFirm];
extern Worker worker[NWorker];
extern Holder holder[NHolder];
extern Env env;
extern long seed;

void preF(int time){
	int i;

	for(i=0;i<NFirm;i++){
		firm[i].demand[time] = 0.0;
		firm[i].invest[time] = 0.0;
	}
}

void investF(int time){
	int h,i,t,choice;
	double b,aveX,v[NFirm];
	t = time - 1;
	aveX = 0.0;
	for(i=0;i<NFirm;i++) aveX += (firm[i].demand[t]);
	aveX /= NFirm;
	for(h=0;h<NHolder;h++){
		b = holder[h].csr;
		for(i=0;i<NFirm;i++)
			v[i] = maxF(0.0, env.iner + (firm[i].demand[t] - aveX)
				 / aveX + b * firm[i].csr);
		choice = choiceVecF(v);
		holder[h].choice[time] = choice;
		(firm[choice].invest[time]) += (holder[h].invest);
	}
	for(i=0;i<NFirm;i++)
		firm[i].stock[time] = (1.0 - env.depr) * firm[i].stock[t]
					+ firm[i].invest[time];
}

void jobF(int time){
	int h,i,t,choice;
	double b,aveW,v[NFirm];

	t = time - 1;
	aveW = 0.0;
	for(i=0;i<NFirm;i++) aveW += (firm[i].wage[t]);
	aveW /= NFirm;
	for(h=0;h<NWorker;h++){
		b = worker[h].csrW;
		for(i=0;i<NFirm;i++)
			v[i] = maxF(0.0,(env.iner + firm[i].wage[t] - aveW)
				 / aveW + b * firm[i].csr);
		choice = choiceVecF(v);
		worker[h].choice[time] = choice;
		worker[h].effort[time] = maxF(0.0, env.eff + worker[h].abl
			+ (firm[choice].wage[t] - aveW) / aveW
			+ firm[choice].csr * b);
	}
}

void productF(int time){
	int i,w,num;
	double sum,wage;

	for(i=0;i<NFirm;i++){
		num = 0;
		sum = 0.0;
		for(w=0;w<NWorker;w++) if(worker[w].choice[time] == i){
			num++;
			sum += worker[w].effort[time];
		}
		firm[i].quantity[time]
			 = (1.0 - env.upcsr * firm[i].csr)
			 * env.marginal
			 * maxF(0.0,sum + env.beta * firm[i].stock[time]);
		if(num != 0) wage = firm[i].quantity[time] / num;
		else wage = 0.0;
		firm[i].wage[time] = wage;
		for(w=0;w<NWorker;w++) if(worker[w].choice[time] == i)
			worker[w].wage[time] = wage;
	}
}

void consumpF(int time){
	int h,i,choice;
	double b,aveQ,v[NFirm];


	aveQ = 0.0;
	for(i=0;i<NFirm;i++) aveQ += (firm[i].quantity[time]);
	aveQ /= NFirm;

	for(h=0;h<NWorker;h++){
		b = worker[h].csrC;
		for(i=0;i<NFirm;i++)
			v[i] = maxF(0.0, env.iner + b * firm[i].csr)
			 + (firm[i].quality[time] - aveQ) / aveQ;
		choice = choiceVecF(v);
		worker[h].consump[time] = choice;
		(firm[choice].demand[time]) += (worker[h].wage[time]);
	}
}

void cutCsrF(){
	int i;
	for(i=0;i<NWorker;i++){
		worker[i].csrW = 0.0;
		worker[i].csrC = 0.0;
	}
	for(i=0;i<NHolder;i++) holder[i].csr = 0.0;

}
